import { ResourceOwnerClaims } from 'byu-jwt'
import { Request, Response } from 'express'

/**
 * Create a client handler with the given configuration
 *
 * @param configObject
 */
export function clientHandler (configObject?: ConfigObject): ClientHandler;

/**
 * Configuration options for the client handler
 */
type ConfigObject = {
    // Whether the request is from an AWS Lambda
    lambda?: boolean,

    // The Express request being handled
    req?: Request,

    // The Express response being handled
    res?: Response,

    // The context within which the request is handled
    // (ex. `/advisement`)
    apiContext?: string
}

/**
 * A handler for the Express framework
 *
 * It handles getting information from the request and sending
 * information through the response, as well as some aspects
 * of authentication and authorization
 */
export interface ClientHandler {
    /**
     * Validate the JWT included in the request
     * @async
     */
    validateJWT(): Promise<void>

    /**
     * Send a response with the given status code and body
     *
     * @param statusCode
     * @param body
     */
    send(statusCode: number, body: any)

    /**
     * Retrieve the body from the request
     */
    body(): any

    /**
     * Get the query parameters
     *
     * If `asURI` is `true`, the returned value is the query string
     * (ex. `?foo=true&bar=0`), otherwise, it's the query object
     *
     * @param asURI
     */
    queryParams(asURI: boolean): string | any

    /**
     * Get the value associated with `paramkey` in the path
     *
     * For example, for a path of
     *
     *     `/authorization/:personId/:resourceId`
     *
     * and a request of
     *
     *     `/authorization/123456789/00001111`
     *
     * a call of `pathValue('personId')` would return `123456789`
     *
     * @param paramkey
     */
    pathValue(paramkey: string): any

    /**
     * Retrieve the JWT from the headers
     */
    parseJWT(): string

    /**
     * Attempt to get the identity using the included JWT
     * @async
     */
    parseIdentity(): Promise<ResourceOwnerClaims>

    /**
     * Get the policies associated with the user and the given web resource ID
     *
     * @async
     * @param webResourceId
     */
    parsePolicies(webResourceId: string): Promise<WebResourcePolicies[]>

    /**
     * Determines whether the current user is authorized to call the API
     *
     * @async
     * @param options.webResourceId - the web resource ID associated with the API
     * @param options.policy - the policy required to have authorization
     * @return whether the user can access the API
     */
    authorize(options: AuthorizeOptions): Promise<boolean>

    /**
     * Resolves the given ID to a BYU ID
     *
     * @async
     * @param idKey - can be either Net ID or BYU ID
     */
    resolveToBYUId(idKey: string): Promise<string>

    /**
     * Resolves the given ID to a Person ID
     *
     * @async
     * @param idKey - can be either Net ID or BYU ID
     */
    resolveToPersonId(idKey: string): Promise<string>

    /**
     * Determines whether the given user has been delegated access by
     * another person
     *
     * @async
     * @param delegatedId - the Person ID of the user that was delegated access by `fromId`
     * @param fromId - the Person ID of the user that delegated access to `delegatedId`
     * @param category - the category in which to check for delegated access
     */
    hasAccessDelegation(delegatedId: string, fromId: string, category: string): Promise<WebResourcePolicies[]>
}

export type WebResourcePolicies = {
    creditInstitution: string | null
    personId: string
    webResourceId: string
    informationalArea: string
    effectiveDate: string | null
    expiredDate: string | null
    limitationType: string | null
    limitationValue: string | null
    updateType: string
    clockStartTime: string
    clockEndTime: string
    allowableDomain: string
}

/**
 * An object representing options passed into the `authorize` function
 */
type AuthorizeOptions = {
    webResourceId: string
    policy: string
    updateType?: string
    allowSelf?: boolean
    selfIdKey?: string
}