/*
 * Copyright 2016 Brigham Young University
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

'use strict';

const setEnv        = require('../tests/setEnv')
setEnv.setEnvFromFile('testing.json')

let   clientFactory = require('../index')
const expect        = require('chai').expect
const Promise       = require('bluebird')
const co            = Promise.coroutine
const Request       = require('request-promise')

describe('client handler', function()
{
    it('resolve to person id', function()
    {
        const client = clientFactory.clientHandler({req: {headers: {}, params: {id: 'mrg2'}}})

        return client.resolveToPersonId('id')
            .then(function (personId)
            {
                expect(personId).equal("787202462")
            })
            .catch (function (e)
            {
                expect(e.message).equal("unable to establish identity")
            })
    });
    it('has access delegation', function()
    {
        const client = clientFactory.clientHandler({req: {headers: {}}})

        return client.hasAccessDelegation('393279252', '658280902', 'Academic')
            .then(function (isAuthorized)
            {
                expect(isAuthorized[0].informationalArea).equal("DELEGATION")
            })
            .catch (function (e)
            {
                expect(e.message).equal("unable to establish identity")
            })
    });
    it('config', function()
    {
        const client = clientFactory.clientHandler()
        expect(client).to.contain.keys('lambda', 'res', 'req', 'policyCache', 'response', 'apiContext')
    });

    it('queryParameters as URI', function()
    {
        const client = clientFactory.clientHandler({req: {}})
        client.req.url = "https://api.byu.edu/abc/?yearTerm=15&programId=12345"
        expect(client.queryParams(true)).to.equal("yearTerm=15&programId=12345")

        client.req.url = "https://api.byu.edu/abc"
        expect(client.queryParams(true)).to.equal("")

        client.req.url = "/curriculum/academicPrograms?yearTerm=20175&limit=1"

        expect(client.queryParams(true)).to.equal("yearTerm=20175&limit=1")
    });

    it('authorize no identity', function()
    {
        const client = clientFactory.clientHandler({req: {headers: {}}})

        return client.authorize({webResourceId: "00002732", policy: "REGADDDROPVIEW"})
            .then(function (isAuthorized)
            {
                expect(isAuthorized).equal(true)
            })
            .catch (function (e)
            {
                expect(e.message).equal("unable to establish identity")
            })
    });

    it('authorize mock identity', function()
    {
        const client = clientFactory.clientHandler()
        //mock identity
        client.identity = {netId: 'mrg2'}

        return client.authorize({webResourceId: "00002732", policy: "REGADDDROPVIEW"})
            .then(function (isAuthorized)
            {
                expect(isAuthorized).equal(true)
            })
    });

    it('not authorized mock identity', function()
    {
        const client = clientFactory.clientHandler()
        //mock identity
        client.identity = {netId: 'mrg2'}

        return client.authorize({webResourceId: "00002732", policy: "REGADDDROPX"})
            .then(function (isAuthorized)
            {
                expect(isAuthorized).equal(false)
            })
    });

    it('not authorized mock identity on updateType', function()
    {
        const client = clientFactory.clientHandler()
        //mock identity
        client.identity = {netId: 'mrg2'}

        return client.authorize({webResourceId: "00002732", policy: "REGADDDROPVIEW", updateType: "X"})
            .then(function (isAuthorized)
            {
                expect(isAuthorized).equal(false)
            })
    });

    it('authorize mock identity with updateType', function()
    {
        const client = clientFactory.clientHandler()
        //mock identity
        client.identity = {netId: 'mrg2'}

        return client.authorize({webResourceId: "00002732", policy: "REGADDDROPVIEW", updateType: "D"})
            .then(function (isAuthorized)
            {
                expect(isAuthorized).equal(true)
            })
    });

    it('authorize self on netId', function()
    {
        const client = clientFactory.clientHandler({req: {params: {id: 'mrg2'}}})
        //mock identity
        client.identity = {netId: 'mrg2'}

        return client.authorize({allowSelf: true, selfIdKey: "id"})
            .then(function (isAuthorized)
            {
                expect(isAuthorized).equal(true)
            })
    });

    it('fail authorize self on netId', function()
    {
        const client = clientFactory.clientHandler({req: {params: {id: 'mrg2x'}}})
        //mock identity
        client.identity = {netId: 'mrg2'}

        return client.authorize({allowSelf: true, selfIdKey: "id"})
            .then(function (isAuthorized)
            {
                expect(isAuthorized).equal(false)
            })
    });

    it('authorize self on byuId', function()
    {
        const client = clientFactory.clientHandler({req: {params: {id: '123456789'}}})
        //mock identity
        client.identity = {byuId: '123456789'}

        return client.authorize({allowSelf: true, selfIdKey: "id"})
            .then(function (isAuthorized)
            {
                expect(isAuthorized).equal(true)
            })
    });

    it('fail authorize self on byuId', function()
    {
        const client = clientFactory.clientHandler({req: {params: {id: '123456788'}}})
        //mock identity
        client.identity = {byuId: '123456789'}

        return client.authorize({allowSelf: true, selfIdKey: "id"})
            .then(function (isAuthorized)
            {
                expect(isAuthorized).equal(false)
            })
    });

    it('fail authorize on CESAdmissions', function()
    {
        const client = clientFactory.clientHandler({req: {params: {id: 'trevash'}}})
        //mock identity
        client.identity = {netId: 'mrg2'}

        return client.authorize({allowSelf: true,
            selfIdKey: "id",
            webResourceId: "12345678",
            policy: "CESAdmissions"
        })
            .then(function (isAuthorized)
            {
                expect(isAuthorized).equal(false)
            })
    });

    it('authorize on CESAdmissions', function()
    {
        const client = clientFactory.clientHandler({req: {params: {id: 'trevash'}}})
        //mock identity
        client.identity = {netId: 'mrg2'}

        return client.authorize({allowSelf: true,
            selfIdKey: "id",
            webResourceId: "00001258",
            policy: "PERSON"
        })
            .then(function (isAuthorized)
            {
                expect(isAuthorized).equal(true)
            })
    });

    it('resolveToBYUId is byuId', function()
    {
        const client = clientFactory.clientHandler({req: {params: {id: '123456789'}}})
        //mock identity
        client.identity = {netId: 'mrg2'}

        return client.resolveToBYUId("id")
            .then(function (byuId)
            {
                expect(byuId).equal("123456789")
            })
    });

    it('resolveToBYUId is netId', function()
    {
        const client = clientFactory.clientHandler({req: {params: {id: 'trevash'}}})
        //mock identity
        client.identity = {netId: 'mrg2'}

        return client.resolveToBYUId("id")
            .then(function (byuId)
            {
                expect(byuId).equal("964386942")
            })
    });

    it('resolveToBYUId is unknow identifier', function()
    {
        const client = clientFactory.clientHandler({req: {params: {id: 'xxxyyy999'}}})
        //mock identity
        client.identity = {netId: 'mrg2'}

        return client.resolveToBYUId("id")
            .then(function (byuId)
            {
                expect(byuId).equal("964386942")
            })
            .catch(function (err)
            {
                console.log(err)
                expect(err).to.contain.keys('statusCode', 'message')
                expect(err.statusCode).equal(409)
            })
    });

    it('parse body', function()
    {
        const client = clientFactory.clientHandler({req: {body: {id: 'xxxyyy999'}}})

        expect(client.body().id).equal("xxxyyy999")
    });
});
